#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

'''
gr-satellites demodulators components

The deframers transform IQ or real samples into soft symbols. Only
demodulation is done, with no packet boundary detection.

The input to these hierarchical blocks is a stream of samples
and the output is a stream of soft symbols.
'''

from .afsk_demodulator import afsk_demodulator
from .bpsk_demodulator import bpsk_demodulator
from .fsk_demodulator import fsk_demodulator
